"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = useCreateReducerWithEffect;
exports.SideEffect = exports.UpdateWithSideEffect = exports.NoUpdate = exports.Update = void 0;

var _react = require("react");

const NO_UPDATE_SYMBOL = Symbol("NO_UPDATE_SYMBOL");

const Update = newState => ({
  newState
});

exports.Update = Update;

const NoUpdate = () => NO_UPDATE_SYMBOL;

exports.NoUpdate = NoUpdate;

const UpdateWithSideEffect = (newState, newSideEffect) => ({
  newState,
  newSideEffect
});

exports.UpdateWithSideEffect = UpdateWithSideEffect;

const SideEffect = newSideEffect => ({
  newSideEffect
});

exports.SideEffect = SideEffect;

async function executeSideEffects({
  sideEffects,
  state,
  dispatch
}) {
  let cancelFuncs = [];

  if (sideEffects) {
    while (sideEffects.length) {
      const sideEffect = sideEffects.shift();
      const cancel = sideEffect(state, dispatch);

      if (cancel && typeof cancel === "function") {
        cancelFuncs.push(cancel);
      }
    }
  }

  return Promise.resolve(cancelFuncs);
}

function finalReducer(reducer) {
  return function (state, action) {
    if (action === NO_UPDATE_SYMBOL) {
      return state;
    }

    let {
      newState,
      newSideEffect
    } = reducer(state.state, action);
    const newSideEffects = Array.isArray(newSideEffect) ? newSideEffect : [newSideEffect];
    newSideEffects && newSideEffects.length ? [...state.sideEffects, ...newSideEffect] : state.sideEffects;
    return {
      state: newState || state.state,
      sideEffects: newSideEffects
    };
  };
}

function useCreateReducerWithEffect(reducer, initialState, init) {
  const [{
    state,
    sideEffects
  }, dispatch] = (0, _react.useReducer)(finalReducer(reducer), {
    state: initialState,
    sideEffects: []
  }, init);
  let cancelFuncs = (0, _react.useRef)([]);
  (0, _react.useEffect)(() => {
    if (sideEffects.length) {
      async function asyncEffects() {
        async function runSideEffects() {
          const cancels = await executeSideEffects({
            sideEffects,
            state,
            dispatch
          });
          return cancels;
        }

        const cancels = await runSideEffects();
        cancelFuncs.current = cancels;
      }

      asyncEffects();

      if (cancelFuncs.current.length) {
        cancelFuncs.current.forEach(func => {
          func(state);
          cancelFuncs.current = [];
        });
      }
    }
  }, [sideEffects]); //eslint-disable-line

  return [state, dispatch];
}